/*
 * Linux_OSCrash.c
 *
 * (C) Copyright IBM Corp. 2009
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author:       Sachin Sant <sachinp@in.ibm.com>
 *
 * Description:
 * Implements resource access functionality for the class Linux_OSCrash.
 */

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include "OSBase_Common.h"

#include "Linux_OSCrash.h"

/*
 * Record OS Crash Date.
 */
static char * get_os_crashdate(void)
{
	char **tmpbuf = NULL;
	char *cdate_str = NULL;
	char *ptr = NULL;
	int rc = 0, buflen = 0;

	_OSBASE_TRACE(3,("--- get_os_crashdate called"));

	rc = runcommand("last -R |grep -m 1 -n crash| awk '{ print $3 $4 $5 }'",
			NULL,
			&tmpbuf,
			NULL);
	if (rc == 0 && tmpbuf[0] != NULL) {
		buflen = strlen(tmpbuf[0]) + 1;
		cdate_str = (char *)calloc(1, buflen);
		strncpy(cdate_str, tmpbuf[0], buflen);
		if ((ptr = strchr(cdate_str, '\n')))
			*ptr = '\0';
	} else {
		cdate_str = (char *)calloc(1, 14);
		strncpy(cdate_str, "Not available", 14);
	}
	
	if (tmpbuf)
		freeresultbuf(tmpbuf);
		
	_OSBASE_TRACE(3,("--- get_os_crashdate exited"));
	return cdate_str;
}

/*
 * Record OS Crash Time.
 */
static char *get_os_crashtime(void)
{
	char *ctime_str = NULL;
	char **tmpbuf = NULL;
	char *ptr = NULL;
	int rc = 0, buflen = 0;

	_OSBASE_TRACE(3,("--- get_os_crashtime called"));

	rc = runcommand("last -R | grep -m 1 -n crash | awk '{ print $6 }'",
			NULL,
			&tmpbuf,
			NULL);
	if (rc == 0 && tmpbuf[0] != NULL) {
		buflen = strlen(tmpbuf[0]) + 1;
		ctime_str = (char *)calloc(1, buflen);
		strncpy(ctime_str, tmpbuf[0], buflen);
		if ((ptr = strchr(ctime_str, '\n')))
			*ptr = '\0';
	} else {
		ctime_str = (char *)calloc(1, 14);
		strncpy(ctime_str, "Not available", 14);
	}

	if (tmpbuf)
		freeresultbuf(tmpbuf);

	_OSBASE_TRACE(3,("--- get_os_crashtime exited"));
	return ctime_str;
}

/* Returns hostname */
static char *get_host_name(void)
{
	char *host = NULL, *chost_str = NULL;
	char *domain = NULL, *ptr = NULL;
	char **hdout = NULL;
	int rc = 0;

	_OSBASE_TRACE(3,("--- get_host_name() called"));

	host = (char *)calloc(1,255);
	if (gethostname(host, 255) == -1) {
		chost_str = calloc(1,14);
		strcpy(chost_str, "Not available");
		goto out;
	}

	if( strchr( host , '.') == NULL ) {
	/* get domain name */
		rc=runcommand("/bin/dnsdomainname",NULL,&hdout,NULL);
		if (rc == 0 && hdout != NULL) {
			if (hdout[0]) {
				domain = strdup(hdout[0]);
				ptr = strchr(domain, '\n');
				*ptr = '\0';
			}
			freeresultbuf(hdout);
		}
	}

	if( strlen(host) ) {
		if( !domain ) {
			chost_str = calloc(1,(strlen(host)+1));
			strcpy( chost_str, host);
		} else {
			chost_str = calloc(1,(strlen(host)+strlen(domain)+2));
			strcpy( chost_str, host);
			strcat( chost_str, ".");
			strcat( chost_str, domain );
		}
	}

	if(domain) free(domain);
out:
	if(host) free(host);

	_OSBASE_TRACE(3,("--- get_host_name() exited"));

	return chost_str;
}
/* Returns a structure, which contains necessary information about Crash 
 * Event
 * return 0 on sucsess.
 * return 1 on failure.
 */
int get_Linux_OSCrash_data( struct oscrash_data **ptr )
{
	_OSBASE_TRACE(3,("--- get_Linux_OSCrash_data called"));

	(*ptr) = calloc(1, sizeof(struct oscrash_data));
	if (*ptr == NULL) {
		_OSBASE_TRACE(3,
			("-- Allocation of struct oscrash_data failed"));
		_OSBASE_TRACE(3,("--- get_Linux_OSCrash_data exited"));
		return -1;
	}

	(*ptr)->hostName = get_host_name();
	(*ptr)->crashDate = get_os_crashdate();
	(*ptr)->crashTime = get_os_crashtime();

	if (!strcmp((*ptr)->crashDate, "Not available")) {
		free_Linux_OSCrash_data(*ptr);
		_OSBASE_TRACE(3,
			("-- No crash data available"));
		_OSBASE_TRACE(3,("--- get_Linux_OSCrash_data exited"));
		return 1;
	}

	_OSBASE_TRACE(4,("================================================"));
	_OSBASE_TRACE(4,("  Host Name : %s", (*ptr)->hostName));
	_OSBASE_TRACE(4,(" Crash Date : %s", (*ptr)->crashDate));
	_OSBASE_TRACE(4,(" Crash time : %s", (*ptr)->crashTime));
	_OSBASE_TRACE(4,("================================================"));

	get_Linux_CrashDumpCapture_data(ptr);

	_OSBASE_TRACE(3,("--- get_Linux_OSCrash_data exited"));

	return 0;
}

/*
 * Free all resources.
 */
void free_Linux_OSCrash_data(struct oscrash_data *ptr)
{
	_OSBASE_TRACE(3,("--- free_Linux_OSCrash_data called"));
	
	if (ptr == NULL)
		return;
	if (ptr->hostName)
		free(ptr->hostName);		
	if (ptr->crashDate)
		free(ptr->crashDate);		
	if (ptr->crashTime)
		free(ptr->crashTime);
	if (ptr->dumpFileLocation)
		free(ptr->dumpFileLocation);
	if (ptr->dumpFileName)
		free(ptr->dumpFileName);
	if (ptr->dumpCaptureDateTime)
		free(ptr->dumpCaptureDateTime);
	free(ptr);

	_OSBASE_TRACE(3,("--- free_Linux_OSCrash_data exited"));
}		
