/*
 * Linux_OSCrashProvider.c
 *
 * (C) Copyright IBM Corp. 2009
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Author:       Sachin Sant <sachinp@in.ibm.com>
 *
*/

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include "cmpidt.h"
#include "cmpimacs.h"

#include "OSBase_Common.h"

#include "Linux_OSCrash.h"

static int enabled = 1;
static int numSubscription = 0;
static int classLoaded = 0;

static CMPIInstance *instCreation = NULL, *instProcess = NULL;
static const CMPIBroker *broker;
static const char *namespace = NULL;

static int fill_OSCrashdata(void)
{
	int ret;
	struct oscrash_data *oscrashdata = NULL;

	_OSBASE_TRACE(2,("--- fill_OSCrashdata() called"));

	ret = get_Linux_OSCrash_data(&oscrashdata);
	if (ret)
		return -1;

	CMSetProperty(instProcess, "hostName",
		oscrashdata->hostName, CMPI_chars);
	CMSetProperty(instProcess, "crashDate",
		oscrashdata->crashDate, CMPI_chars);
	CMSetProperty(instProcess, "crashTime",
		oscrashdata->crashTime, CMPI_chars);
	CMSetProperty(instProcess, "dumpFileLocation",
		oscrashdata->dumpFileLocation, CMPI_chars);
	CMSetProperty(instProcess, "dumpFileName",
		oscrashdata->dumpFileName, CMPI_chars);
	CMSetProperty(instProcess, "dumpCaptureDateTime",
		oscrashdata->dumpCaptureDateTime, CMPI_chars);
	CMSetProperty(instProcess, "dumpFileSize",
		&(oscrashdata->dumpFileSize), CMPI_uint64);

	free_Linux_OSCrash_data(oscrashdata);

	_OSBASE_TRACE(2,("--- fill_OSCrashdata() exited"));

	return 0;
}

static int is_LinuxOSCrashClassLoaded( const char *_Classname,
					const CMPIObjectPath *path)
{
	CMPIStatus rc;
	CMPIObjectPath *op = NULL;

	_OSBASE_TRACE(2,("--- is_LinuxOSCrashClassLoaded() called"));

	if (enabled == 0)
		goto exit;

	if (classLoaded == 1) {
		_OSBASE_TRACE(2,("--- is_LinuxOSCrashClassLoaded() exited"));
		return 1;
	}

	namespace = strdup(CMGetCharPtr(CMGetNameSpace(path, &rc)));

	if( !get_system_name() ) {
		CMSetStatusWithChars( broker, &rc,
				CMPI_RC_ERR_FAILED, "no host name found" );
		_OSBASE_TRACE(3,
			("-- is_LinuxOSCrashClassLoaded() failed : %s",
			CMGetCharPtr(rc.msg)));
		goto exit;
	}

	for (;;) {
		op = CMNewObjectPath( broker,
				CMGetCharPtr(CMGetNameSpace(path,&rc)),
				"CIM_InstCreation", &rc );
		if( CMIsNullObject(op) ) {
			CMSetStatusWithChars( broker, &rc,
				CMPI_RC_ERR_FAILED,
				"Create CMPIObjectPath failed." ); 
			_OSBASE_TRACE(2,
				("-- is_LinuxOSCrashClassLoaded() failed : %s",
			CMGetCharPtr(rc.msg)));
			goto exit;
		}

		instCreation = CMNewInstance(broker, op, &rc);
		if( CMIsNullObject(instCreation) ) {
			CMSetStatusWithChars( broker, &rc,
				CMPI_RC_ERR_FAILED,
				"Create CMPIInstance failed." );
			_OSBASE_TRACE(2,
				("-- is_LinuxOSCrashClassLoaded() failed : %s",
			CMGetCharPtr(rc.msg)));
			goto exit;
		}

		instCreation = CMClone(instCreation, &rc);

		CMSetClassName(op, _Classname);

		instProcess = CMNewInstance(broker, op, &rc);
		if( CMIsNullObject(instProcess) ) {
			CMSetStatusWithChars( broker, &rc,
				CMPI_RC_ERR_FAILED,
				"Create CMPIInstance failed." );
			_OSBASE_TRACE(2,
				("-- is_LinuxOSCrashClassLoaded() failed : %s",
			CMGetCharPtr(rc.msg)));
			goto exit;
		}
		
		instProcess = CMClone(instProcess, &rc);
		classLoaded = 1;

		_OSBASE_TRACE(2,("--- is_LinuxOSCrashClassLoaded() exited"));

		return 1;

	}

exit:
	enabled = 0;

	_OSBASE_TRACE(2,("--- is_LinuxOSCrashClassLoaded() exited"));

	return 0;
}

CMPIStatus Linux_OSCrashProviderActivateFilter(
					CMPIIndicationMI *mi,
					const CMPIContext *ctx,
					const CMPISelectExp *filter,
					const char *className,
					const CMPIObjectPath *path,
					CMPIBoolean firstActivation)
{
	CMPIStatus rc;

	_OSBASE_TRACE(2,("-- Linux_OSCrashProviderActivateFilter() called"));
	CMSetStatus(&rc, CMPI_RC_OK);

	if (strcasecmp(className, "Linux_OSCrash") == 0) {

		if (is_LinuxOSCrashClassLoaded(className, path))
			numSubscription++;
	} else {
		CMSetStatusWithChars( broker, &rc, CMPI_RC_ERR_FAILED,
			"OS Crash Provider ClassName match failed");
		_OSBASE_TRACE(2,
			("OS Crash Provider ClassName match failed"));
	}

	_OSBASE_TRACE(2,("-- Linux_OSCrashProviderActivateFilter() exited"));

	return rc;
}

CMPIStatus Linux_OSCrashProviderAuthorizeFilter(
					CMPIIndicationMI *mi,
					const CMPIContext *ctx,
					const CMPISelectExp *filter,
					const char *className,
					const CMPIObjectPath *obj,
					const char *owner)
{
	_OSBASE_TRACE(2,("-- Linux_OSCrashProviderAuthorizeFilter called"));

	_OSBASE_TRACE(2,("-- Linux_OSCrashProviderAuthorizeFilter exited"));
	
	CMReturn(CMPI_RC_OK);
}

CMPIStatus Linux_OSCrashProviderDeActivateFilter(
					CMPIIndicationMI *mi,
					const CMPIContext *ctx,
					const CMPISelectExp *filter,
					const char *className,
					const CMPIObjectPath *path,
					CMPIBoolean lastActivation)
{
	_OSBASE_TRACE(2,("-- Linux_OSCrashProviderDeActivate Filter called"));

	if (strcasecmp(className, "Linux_OSCrash") == 0)
		if (numSubscription > 0)
			numSubscription--;

	_OSBASE_TRACE(2,("-- Linux_OSCrashProviderDeActivate Filter exited"));
	
	CMReturn(CMPI_RC_OK);
}

CMPIStatus Linux_OSCrashProviderMustPoll(
					CMPIIndicationMI *mi,
					const CMPIContext *ctx,
					const CMPISelectExp *filter,
					const char *className,
					const CMPIObjectPath *path)
{
	_OSBASE_TRACE(2,("-- Linux_OSCrashProviderMust Poll called"));

	_OSBASE_TRACE(2,("-- Linux_OSCrashProviderMust Poll exited"));

	CMReturn(CMPI_RC_OK);
}

CMPIStatus Linux_OSCrashProviderDisableIndications(
					CMPIIndicationMI *mi,
					const CMPIContext *ctx)
{
	_OSBASE_TRACE(2,("-- Linux_OSCrashProviderDisable Indications called"));

	enabled = 0;

	_OSBASE_TRACE(2,("-- Linux_OSCrashProviderDisable Indications exited"));

	CMReturn(CMPI_RC_OK);
}

CMPIStatus Linux_OSCrashProviderEnableIndications(
					CMPIIndicationMI *mi,
					const CMPIContext *ctx)
{
	_OSBASE_TRACE(2,("-- Linux_OSCrashProviderEnable Indications called"));

	enabled = 1;
	if (fill_OSCrashdata()) {
		_OSBASE_TRACE(2,("Fill OS Crash Provider data failed"));
	} else {
		CBDeliverIndication(broker, ctx, namespace, instProcess);
		_OSBASE_TRACE(2,("OS Crash Provider Indication delivered"));
	}

	_OSBASE_TRACE(2,("-- Linux_OSCrashProviderEnable Indications exited"));

	CMReturn(CMPI_RC_OK);
}

CMPIStatus Linux_OSCrashProviderIndicationCleanup(
					CMPIIndicationMI *mi,
					const CMPIContext *ctx,
					CMPIBoolean terminate)
{
	_OSBASE_TRACE(2,("-- Linux_OSCrashProviderIndicationCleanup() called"));

	_OSBASE_TRACE(2,("-- Linux_OSCrashProviderIndicationCleanup() exited"));

        CMReturn(CMPI_RC_OK);
}

//------------------------------------------------------------------------
// Provider Factory Stubs
//-----------------------------------------------------------------------

CMIndicationMIStub(Linux_OSCrashProvider,
		Linux_OSCrashProvider,
		broker,
		CMNoHook);
